@@ -312,29 +312,60 @@ impl Proof {
         &self,
         clauses: &Vec<Clause>,
         pred_kind: &HashMap<Predicate, analysis::Kind>,
+        compact: bool,
     ) -> impl TreeItem {
+        fn find_matching_end(
+            operator_start: &Literal,
+            start_index: usize,
+            children: &[Proof],
+        ) -> Option<usize> {
+            for i in start_index + 1..children.len() {
+                let child = &children[i];
+                if let ClauseId::Builtin(b) = &child.clause {
+                    if b.predicate.naive_predicate_kind().is_logic()
+                        && b.predicate.is_operator()
+                        && b.predicate.0.ends_with("_end")
+                        && b.predicate.0.replace("_end", "_begin") == operator_start.predicate.0
+                        && b.args.last() == operator_start.args.last()
+                    {
+                        return Some(i);
+                    }
+                }
+            }
+            None
+        }
+
         fn dfs(
             p: &Proof,
             clauses: &Vec<Clause>,
             builder: &mut TreeBuilder,
             pred_kind: &HashMap<Predicate, analysis::Kind>,
+            compact: bool,
         ) {
-            for child in &p.children {
+            let mut prev_scope_start_index = 0;
+            let mut prev_scope_end_index = 0;
+            let mut dont_close: HashSet<usize> = HashSet::new();
+
+            for (i, child) in p.children.iter().enumerate() {
                 match &child.clause {
                     ClauseId::Rule(rid) => {
-                        let s = clauses[*rid].head.substitute(&child.valuation).to_string();
-                        builder.begin_child(format!(
-                            "{}",
-                            if pred_kind.get(&clauses[*rid].head.predicate)
-                                == Some(&analysis::Kind::Image)
-                            {
-                                s.cyan()
-                            } else {
-                                s.normal()
-                            },
-                        ));
-                        dfs(&child, clauses, builder, pred_kind);
-                        builder.end_child();
+                        if !compact {
+                            let s = clauses[*rid].head.substitute(&child.valuation).to_string();
+                            builder.begin_child(format!(
+                                "{}",
+                                if pred_kind.get(&clauses[*rid].head.predicate)
+                                    == Some(&analysis::Kind::Image)
+                                {
+                                    s.cyan()
+                                } else {
+                                    s.normal()
+                                },
+                            ));
+                        }
+                        dfs(&child, clauses, builder, pred_kind, compact);
+                        if !compact {
+                            builder.end_child();
+                        }
                     }
                     ClauseId::Query => {
                         builder.add_empty_child("query".to_string());
@@ -355,14 +386,29 @@ impl Proof {
                         crate::analysis::Kind::Logic => {
                             if b.predicate.is_operator() {
                                 if b.predicate.0.ends_with("_begin") {
-                                    builder.begin_child("(".to_string());
+                                    let curr_scope_end_index = find_matching_end(b, i, &p.children).expect("begin operator has an end");
+
+                                    let curr_scope_start_index = i;
+                                    if prev_scope_start_index + 1 == curr_scope_start_index
+                                        && curr_scope_end_index + 1 == prev_scope_end_index {
+                                        // if we are directly inside the previous scope, we won't need to start a new scope
+                                        dont_close.insert(prev_scope_end_index);
+                                    } else {
+                                        builder.begin_child("(".to_string());
+                                    }
+                                    prev_scope_start_index = i;
+                                    prev_scope_end_index = curr_scope_end_index;
                                 } else {
-                                    builder.end_child();
+                                    if !dont_close.contains(&i) {
+                                        builder.end_child();
+                                    }
+
                                     // HACK: Because we want to print the operators *after* their scope,
                                     //       we add a leaf node here. It will look as expected in the term
                                     //       but may not make sense as a DAG.
                                     builder.add_empty_child(format!(
-                                        ")::{}",
+                                        "{}::{}",
+                                        if dont_close.contains(&i) { " " } else { ")" },
                                         b.substitute(&child.valuation).unmangle().to_string()
                                     ));
                                 }
@@ -375,7 +421,7 @@ impl Proof {
         }
 
         let mut builder = TreeBuilder::new("".to_string());
-        dfs(self, clauses, &mut builder, pred_kind);
+        dfs(self, clauses, &mut builder, pred_kind, compact);
 
         builder.build()
     }
@@ -384,8 +430,9 @@ impl Proof {
         &self,
         clauses: &Vec<Clause>,
         pred_kind: &HashMap<Predicate, analysis::Kind>,
+        compact: bool,
     ) -> io::Result<()> {
-        print_tree(&self.get_tree(clauses, pred_kind))
+        print_tree(&self.get_tree(clauses, pred_kind, compact))
     }
 }
 
@@ -1171,7 +1218,10 @@ pub fn tree_from_modusfile(
 
 #[cfg(test)]
 mod tests {
-    use crate::modusfile::{Expression, ModusTerm};
+    use crate::{
+        logic::SpannedPosition,
+        modusfile::{Expression, FormatStringFragment, ModusTerm},
+    };
 
     use super::*;
     use serial_test::serial;
@@ -1220,6 +1270,27 @@ mod tests {
         ));
     }
 
+    #[test]
+    #[serial]
+    fn simple_solving_with_escape_chars() {
+        let goal: Goal<logic::IRTerm> = vec!["a".parse().unwrap()];
+        let clauses: Vec<logic::Clause> = vec![
+            r#"a :- b("\n")."#.parse().unwrap(),
+            logic::Clause {
+                head: r#"b("\n")"#.parse().unwrap(),
+                body: vec![],
+            },
+        ];
+        let tree = sld(&clauses, &goal, 10, true).tree;
+        let solutions = solutions(&tree);
+        assert_eq!(solutions.len(), 1);
+
+        assert!(contains_ignoring_position(
+            &solutions,
+            &vec![r#"a"#.parse::<logic::Literal>().unwrap()]
+        ));
+    }
+
     #[test]
     #[serial]
     fn simple_negation_solving() {
@@ -1488,13 +1559,7 @@ mod tests {
             positive: true,
             position: None,
             predicate: Predicate("base_image".into()),
-            args: vec![ModusTerm::FormatString {
-                position: logic::SpannedPosition {
-                    offset: 11,
-                    length: 13,
-                },
-                format_string_literal: "alpine${X}".to_owned(),
-            }],
+            args: vec!["f\"alpine${X}\"".parse().unwrap()],
         });
 
         let (_, _, sld_res) = tree_from_modusfile(mf, query, 20, true);
